# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import django
from django.db import models, migrations
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0001_initial'),
    ]

    if django.get_version() >= '1.8':
        operations = [
            migrations.CreateModel(
                name='Template',
                fields=[
                    ('id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                    ('name', models.CharField(
                        help_text="Example: 'flatpages/default.html'", max_length=100, verbose_name='name')),
                    ('content', models.TextField(verbose_name='content', blank=True)),
                    ('creation_date', models.DateTimeField(
                        default=django.utils.timezone.now, verbose_name='creation date')),
                    ('last_changed', models.DateTimeField(
                        default=django.utils.timezone.now, verbose_name='last changed')),
                    ('sites', models.ManyToManyField(
                        to='sites.Site', verbose_name='sites', blank=True)),
                ],
                options={
                    'ordering': ('name',),
                    'db_table': 'django_template',
                    'verbose_name': 'template',
                    'verbose_name_plural': 'templates',
                },
                bases=(models.Model,),
                managers = [
                    ('objects', django.db.models.manager.Manager()),
                    ('on_site', django.contrib.sites.managers.CurrentSiteManager(
                        b'sites')),
                ],
            ),
        ]
    else:
        operations = [
            migrations.CreateModel(
                name='Template',
                fields=[
                    ('id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                    ('name', models.CharField(
                        help_text="Example: 'flatpages/default.html'", max_length=100, verbose_name='name')),
                    ('content', models.TextField(verbose_name='content', blank=True)),
                    ('creation_date', models.DateTimeField(
                        default=django.utils.timezone.now, verbose_name='creation date')),
                    ('last_changed', models.DateTimeField(
                        default=django.utils.timezone.now, verbose_name='last changed')),
                    ('sites', models.ManyToManyField(
                        to='sites.Site', verbose_name='sites', blank=True)),
                ],
                options={
                    'ordering': ('name',),
                    'db_table': 'django_template',
                    'verbose_name': 'template',
                    'verbose_name_plural': 'templates',
                },
                bases=(models.Model,),
            ),
        ]



from django.core import signals
from django.contrib.sites.models import Site
from django.template.defaultfilters import slugify

from dbtemplates.conf import settings


def get_cache_backend():
    """
    Compatibilty wrapper for getting Django's cache backend instance
    """
    try:
        from django.core.cache import _create_cache
    except ImportError:
        # Django < 1.7
        from django.core.cache import get_cache as _get_cache
        return _get_cache(settings.DBTEMPLATES_CACHE_BACKEND)

    cache = _create_cache(settings.DBTEMPLATES_CACHE_BACKEND)
    # Some caches -- python-memcached in particular -- need to do a cleanup at the
    # end of a request cycle. If not implemented in a particular backend
    # cache.close is a no-op
    signals.request_finished.connect(cache.close)
    return cache

cache = get_cache_backend()


def get_cache_key(name):
    current_site = Site.objects.get_current()
    return 'dbtemplates::%s::%s' % (slugify(name), current_site.pk)


def get_cache_notfound_key(name):
    return get_cache_key(name) + '::notfound'


def remove_notfound_key(instance):
    # Remove notfound key as soon as we save the template.
    cache.delete(get_cache_notfound_key(instance.name))


def set_and_return(cache_key, content, display_name):
    # Save in cache backend explicitly if manually deleted or invalidated
    if cache:
        cache.set(cache_key, content)
    return (content, display_name)


def add_template_to_cache(instance, **kwargs):
    """
    Called via Django's signals to cache the templates, if the template
    in the database was added or changed.
    """
    remove_cached_template(instance)
    remove_notfound_key(instance)
    cache.set(get_cache_key(instance.name), instance.content)


def remove_cached_template(instance, **kwargs):
    """
    Called via Django's signals to remove cached templates, if the template
    in the database was changed or deleted.
    """
    cache.delete(get_cache_key(instance.name))



from django import VERSION
from django.template import (Template, TemplateDoesNotExist,
                             TemplateSyntaxError)
from django.utils.importlib import import_module


def get_loaders():
    if VERSION[:2] < (1, 8):
        from django.template.loader import template_source_loaders
        if template_source_loaders is None:
            try:
                from django.template.loader import find_template as finder
            except ImportError:
                from django.template.loader import find_template_source as finder  # noqa
            try:
                source, name = finder('test')
            except TemplateDoesNotExist:
                pass
            from django.template.loader import template_source_loaders
        return template_source_loaders or []
    else:
        from django.template.loader import _engine_list
        loaders = []
        for engine in _engine_list():
            loaders.extend(engine.engine.template_loaders)
        return loaders


def get_template_source(name):
    source = None
    for loader in get_loaders():
        if loader.__module__.startswith('dbtemplates.'):
            # Don't give a damn about dbtemplates' own loader.
            continue
        module = import_module(loader.__module__)
        load_template_source = getattr(
            module, 'load_template_source', None)
        if load_template_source is None:
            load_template_source = loader.load_template_source
        try:
            source, origin = load_template_source(name)
            if source:
                return source
        except NotImplementedError:
            pass
        except TemplateDoesNotExist:
            pass
    if source is None and VERSION[:2] < (1, 2):
        # Django supported template source extraction still :/
        try:
            from django.template.loader import find_template_source
            template, origin = find_template_source(name, None)
            if not hasattr(template, 'render'):
                return template
        except (ImportError, TemplateDoesNotExist):
            pass
    return None


def check_template_syntax(template):
    try:
        Template(template.content)
    except TemplateSyntaxError as e:
        return (False, e)
    return (True, None)
